@@ -40,12 +40,6 @@
 
 #include <gdk/gdkx.h>
 
-#if DEBUG
-#define debug_printf(args...) do { g_print ("debug: " args); } while (0)
-#else
-#define debug_printf(args...) do { /* nothing */ } while (0)
-#endif
-
 #define ANIMATION_UP 0
 #define ANIMATION_DOWN 1
 
@@ -230,10 +224,10 @@ void pull (struct tilda_window_ *tw, enum pull_action action, gboolean force_hid
 
     if (tw->current_state == STATE_DOWN && needsFocus) {
         /**
-        * See tilda_window.c in focus_out_event_cb for an explanation about focus_loss_on_keypress
-        * This conditional branch will only focus tilda but it does not actually pull the window up.
-        */
-        TRACE (g_print("Tilda window not focused but visible\n"));
+         * See tilda_window.c in focus_out_event_cb for an explanation about focus_loss_on_keypress
+         * This conditional branch will only focus tilda but it does not actually pull the window up.
+         */
+        g_debug ("Tilda window not focused but visible");
         gdk_x11_window_set_user_time(gtk_widget_get_window(tw->window),
                 tomboy_keybinder_get_current_event_time());
         tilda_window_set_active(tw);
@@ -297,7 +291,7 @@ static void pull_up (struct tilda_window_ *tw) {
      * Case 2 - Animation off: Just hide the window */
     gtk_widget_hide (GTK_WIDGET(tw->window));
 
-    debug_printf ("pull_up(): MOVED UP\n");
+    g_debug ("pull_up(): MOVED UP");
     tw->current_state = STATE_UP;
 }
 
@@ -376,7 +370,7 @@ static void pull_down (struct tilda_window_ *tw) {
      * not work anymore. This is working for now... */
     tilda_window_set_active (tw);
 
-    debug_printf ("pull_down(): MOVED DOWN\n");
+    g_debug ("pull_down(): MOVED DOWN");
     tw->current_state = STATE_DOWN;
 }
 
