@@ -23,9 +23,6 @@
 G_BEGIN_DECLS
 
 #define TILDA_VERSION PACKAGE_NAME " " PACKAGE_VERSION
-#define QUICK_STRCMP(a, b) (*(a)!=*(b)? \
-    (int) ((unsigned char) *(a) - (unsigned char) *(b)) : \
-    strcmp ((a), (b)))
 
 struct lock_info
 {
@@ -34,8 +31,31 @@ struct lock_info
     gint file_descriptor;
 };
 
-#define GUINT16_TO_FLOAT(color) (color / (double) 0xFFFF)
-#define GUINT16_FROM_FLOAT(value) ((int) (value * 0xFFFF + 0.5))
+typedef struct tilda_cli_options tilda_cli_options;
+
+struct tilda_cli_options {
+    gchar *background_color;
+    gchar *command;
+    gchar *font;
+    gchar *working_dir;
+#ifdef VTE_290
+    gchar *image;
+    gint *transparency;
+#else
+    gint back_alpha;
+#endif
+    gint lines;
+    gint x_pos;
+    gint y_pos;
+    gboolean antialias;
+    gboolean scrollbar;
+    gboolean show_config;
+    gboolean version;
+    gboolean hidden;
+};
+
+#define GUINT16_TO_FLOAT(color) ((color) / (double) 0xFFFF)
+#define GUINT16_FROM_FLOAT(value) ((int) ((value) * 0xFFFF + 0.5))
 
 #define RGB(r,g,b) (r) / (gdouble)G_MAXUINT16, \
                      (g) / (gdouble)G_MAXUINT16, \
@@ -44,3 +64,13 @@ struct lock_info
 G_END_DECLS
 
 #endif
+
+/* Future compatibility with VTE-2.90 */
+#ifndef VTE_290 
+#define vte_terminal_set_colors_rgba vte_terminal_set_colors
+#define vte_terminal_set_color_background_rgba vte_terminal_set_color_background
+#define vte_terminal_set_color_foreground_rgba vte_terminal_set_color_foreground
+#define vte_terminal_set_color_cursor_rgba vte_terminal_set_color_cursor
+#define VteTerminalCursorShape VteCursorShape
+#endif
+
