@@ -19,7 +19,6 @@
 #include "tilda.h"
 #include "tilda_window.h"
 #include "tilda_terminal.h"
-#include "callback_func.h"
 #include "configsys.h"
 #include "wizard.h" /* wizard */
 
@@ -32,7 +31,7 @@
 #include <vte/vte.h>
 #include <string.h>
 
-#define HTTP_REGEXP "(ftp|http)s?://[-a-zA-Z0-9.?$%&/=_~#.,:;+]*"
+#define HTTP_REGEXP "(ftp|http)s?://[\\[\\]-a-zA-Z0-9.?$%&/=_~#.,:;+]*"
 
 GdkRGBA current_palette[TERMINAL_PALETTE_SIZE];
 
@@ -239,7 +238,7 @@ static void window_title_changed_cb (GtkWidget *widget, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     tilda_term *tt = TILDA_TERM(data);
-    gchar *title = get_window_title (widget);
+    gchar *title = tilda_terminal_get_title (data);
     GtkWidget *label;
 
     label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (tt->tw->notebook), tt->hbox);
@@ -254,10 +253,16 @@ static void window_title_changed_cb (GtkWidget *widget, gpointer data)
     }
 
     guint length = (guint) config_getint ("title_max_length");
-
-    if(config_getbool("title_max_length_flag") && strlen(title) > length) {
-        gchar *titleOffset = title + strlen(title) - length;
-        gchar *shortTitle = g_strdup_printf ("...%s", titleOffset);
+    guint title_behaviour = config_getint("title_behaviour");
+    if(title_behaviour && strlen(title) > length) {
+        gchar *shortTitle = NULL;
+        if(title_behaviour == 1) {
+            shortTitle = g_strdup_printf ("%.*s...", length, title);
+        }
+        else {
+            gchar *titleOffset = title + strlen(title) - length;
+            shortTitle = g_strdup_printf ("...%s", titleOffset);
+        }
         gtk_label_set_text (GTK_LABEL(label), shortTitle);
         if (active) {
             gtk_window_set_title (GTK_WINDOW (tt->tw->window), shortTitle);
@@ -710,7 +715,7 @@ static gint tilda_term_config_defaults (tilda_term *tt)
 
     /** Text Properties **/
     vte_terminal_set_allow_bold (VTE_TERMINAL(tt->vte_term), config_getbool ("bold"));
-    gtk_widget_set_double_buffered (tt->vte_term, config_getbool("double_buffer"));
+
     PangoFontDescription *description =
         pango_font_description_from_string (config_getstr ("font"));
     vte_terminal_set_font (VTE_TERMINAL (tt->vte_term), description);
@@ -835,7 +840,7 @@ menu_quit_cb (GSimpleAction *action,
 {
     DEBUG_FUNCTION ("menu_quit_cb");
 
-    gtk_main_quit ();
+    tilda_window_confirm_quit(TILDA_WINDOW(user_data));
 }
 
 static void
@@ -897,56 +902,9 @@ static void popup_menu (tilda_window *tw, tilda_term *tt)
     DEBUG_ASSERT (tw != NULL);
     DEBUG_ASSERT (tt != NULL);
 
-    static const gchar menu_str[] = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
-            "<interface>\n"
-            "  <menu id=\"menu\">\n"
-            "    <section>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_New Tab</attribute>\n"
-            "        <attribute name=\"action\">window.new-tab</attribute>\n"
-            "      </item>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_Close Tab</attribute>\n"
-            "        <attribute name=\"action\">window.close-tab</attribute>\n"
-            "      </item>\n"
-            "    </section>\n"
-            "    <section>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_Copy</attribute>\n"
-            "        <attribute name=\"action\">window.copy</attribute>\n"
-            "      </item>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_Paste</attribute>\n"
-            "        <attribute name=\"action\">window.paste</attribute>\n"
-            "      </item>\n"
-            "    </section> \n"
-            "    <section>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_Toggle Fullscreen</attribute>\n"
-            "        <attribute name=\"action\">window.fullscreen</attribute>\n"
-            "      </item>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_Toggle Searchbar</attribute>\n"
-            "        <attribute name=\"action\">window.searchbar</attribute>\n"
-            "      </item>\n"
-            "    </section> \n"
-            "    <section>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_Preferences</attribute>\n"
-            "        <attribute name=\"action\">window.preferences</attribute>\n"
-            "      </item>\n"
-            "    </section> \n"
-            "    <section>\n"
-            "      <item>\n"
-            "        <attribute name=\"label\" translatable=\"yes\">_Quit</attribute>\n"
-            "        <attribute name=\"action\">window.quit</attribute>\n"
-            "      </item>\n"
-            "    </section> \n"
-            "  </menu>\n"
-            "</interface>";
-
     GtkBuilder *builder = gtk_builder_new();
-    gtk_builder_add_from_string(builder, menu_str, strlen(menu_str), NULL);
+
+    gtk_builder_add_from_resource (builder, "/org/tilda/menu.ui", NULL);
 
     /* Create the action group */
     GSimpleActionGroup *action_group = g_simple_action_group_new();
@@ -1005,7 +963,6 @@ static int button_press_cb (G_GNUC_UNUSED GtkWidget *widget, GdkEventButton *eve
     tilda_term *tt;
     gchar *match;
     gint tag;
-    gint ypad;
     gchar *cmd;
     gchar *web_browser_cmd;
     gboolean ret = FALSE;
@@ -1027,32 +984,41 @@ static int button_press_cb (G_GNUC_UNUSED GtkWidget *widget, GdkEventButton *eve
             break;
         case 1: /* Left Click */
             terminal  = VTE_TERMINAL(tt->vte_term);
-            ypad = gtk_widget_get_margin_bottom(GTK_WIDGET(terminal));
+
+#if VTE_CHECK_VERSION(0, 38, 0)
+            match = vte_terminal_match_check_event (terminal,
+                                                    (GdkEvent *) event,
+                                                    &tag);
+#else
+            gint ypad = gtk_widget_get_margin_bottom(GTK_WIDGET(terminal));
+
+            glong column = (glong) ((event->x - ypad) /
+                                    vte_terminal_get_char_width (terminal));
+            glong row = (glong) ((event->y - ypad) /
+                                 vte_terminal_get_char_height (terminal));
 
             match = vte_terminal_match_check (terminal,
-                    (event->x - ypad) /
-                    vte_terminal_get_char_width (terminal),
-                    (event->y - ypad) /
-                    vte_terminal_get_char_height (terminal),
-                    &tag);
+                                              column,
+                                              row,
+                                              &tag);
+#endif
 
             /* Check if we can launch a web browser, and do so if possible */
             if (match != NULL)
             {
-#if DEBUG
-                g_print ("Got a Left Click -- Matched: `%s' (%d)\n", match, tag);
-#endif
+                g_debug ("Got a Left Click -- Matched: `%s' (%d)", match, tag);
+
                 web_browser_cmd = g_strescape (config_getstr ("web_browser"), NULL);
                 cmd = g_strdup_printf ("%s %s", web_browser_cmd, match);
-#if DEBUG
-                g_print ("Launching command: `%s'\n", cmd);
-#endif
+
+                g_debug ("Launching command: `%s'", cmd);
+
                 ret = g_spawn_command_line_async(cmd, NULL);
 
                 /* Check that the command launched */
                 if (!ret)
                 {
-                    g_printerr (_("Failed to launch the web browser. The command was `%s'\n"), cmd);
+                    g_critical (_("Failed to launch the web browser. The command was `%s'\n"), cmd);
                     TILDA_PERROR ();
                 }
 
@@ -1083,3 +1049,53 @@ gboolean key_press_cb (GtkWidget *widget,
 }
 
 /* vim: set ts=4 sts=4 sw=4 expandtab: */
+
+gchar *tilda_terminal_get_title (tilda_term *tt)
+{
+    DEBUG_FUNCTION ("get_window_title");
+    DEBUG_ASSERT (tt != NULL);
+
+    const gchar *vte_title;
+    gchar *window_title;
+    gchar *initial;
+    gchar *title;
+
+    vte_title = vte_terminal_get_window_title (VTE_TERMINAL (tt->vte_term));
+    window_title = g_strdup (vte_title);
+    initial = g_strdup (config_getstr ("title"));
+
+    /* These are not needed anywhere else. If they ever are, move them to a header file */
+    enum d_set_title { NOT_DISPLAYED, AFTER_INITIAL, BEFORE_INITIAL, REPLACE_INITIAL };
+
+    switch (config_getint ("d_set_title"))
+    {
+        case REPLACE_INITIAL:
+            title = (window_title != NULL) ? g_strdup (window_title)
+                                           : g_strdup (_("Untitled"));
+            break;
+
+        case BEFORE_INITIAL:
+            title = (window_title != NULL) ? g_strdup_printf ("%s - %s", window_title, initial)
+                                           : g_strdup (initial);
+            break;
+
+        case AFTER_INITIAL:
+            title = (window_title != NULL) ? g_strdup_printf ("%s - %s", initial, window_title)
+                                           : g_strdup (initial);
+            break;
+
+        case NOT_DISPLAYED:
+            title = g_strdup (initial);
+            break;
+
+        default:
+            g_printerr (_("Bad value for \"d_set_title\" in config file\n"));
+            title = g_strdup ("");
+            break;
+    }
+
+    g_free (window_title);
+    g_free (initial);
+
+    return title;
+}
