@@ -17,7 +17,6 @@
 
 #include "debug.h"
 #include "tilda.h"
-#include "callback_func.h"
 #include "configsys.h"
 #include "tilda_window.h"
 #include "tilda_terminal.h"
@@ -37,6 +36,12 @@
 
 static tilda_term* tilda_window_get_current_terminal (tilda_window *tw);
 
+static gboolean show_confirmation_dialog (tilda_window *tw,
+                                          const char *message);
+
+static void tilda_window_apply_transparency (tilda_window *tw,
+                                             gboolean status);
+
 static void
 tilda_window_setup_alpha_mode (tilda_window *tw)
 {
@@ -90,10 +95,50 @@ void tilda_window_close_current_tab (tilda_window *tw)
     DEBUG_FUNCTION ("close_current_tab");
     DEBUG_ASSERT (tw != NULL);
 
-    gint pos = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
-    tilda_window_close_tab (tw, pos, FALSE);
+    gboolean can_close = TRUE;
+
+    if (config_getbool ("confirm_close_tab")) {
+        char * message = _("Are you sure you want to close this tab?");
+
+        can_close = show_confirmation_dialog (tw, message);
+    }
+
+    if (can_close) {
+        gint pos = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
+        tilda_window_close_tab (tw, pos, FALSE);
+    }
 }
 
+static gboolean
+show_confirmation_dialog (tilda_window *tw, const char *message)
+{
+    gboolean result;
+
+    GtkWidget *dialog
+            = gtk_message_dialog_new (GTK_WINDOW (tw->window),
+                                      GTK_DIALOG_DESTROY_WITH_PARENT,
+                                      GTK_MESSAGE_QUESTION,
+                                      GTK_BUTTONS_OK_CANCEL,
+                                      NULL);
+
+    gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG(dialog), message);
+
+    gtk_window_set_keep_above (GTK_WINDOW (dialog), TRUE);
+    gint response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+    switch (response) {
+        case GTK_RESPONSE_OK:
+            result = TRUE;
+            break;
+        default:
+            result = FALSE;
+            break;
+    }
+
+    gtk_widget_destroy (dialog);
+
+    return result;
+}
 
 gint tilda_window_set_tab_position (tilda_window *tw, enum notebook_tab_positions pos)
 {
@@ -149,7 +194,7 @@ gint toggle_fullscreen_cb (tilda_window *tw)
     return GDK_EVENT_STOP;
 }
 
-gint toggle_transparency_cb (tilda_window *tw)
+static gint toggle_transparency_cb (tilda_window *tw)
 {
     DEBUG_FUNCTION ("toggle_transparency");
     DEBUG_ASSERT (tw != NULL);
@@ -159,10 +204,22 @@ gint toggle_transparency_cb (tilda_window *tw)
 
 void tilda_window_toggle_transparency (tilda_window *tw)
 {
-    tilda_term *tt;
-    guint i;
     gboolean status = !config_getbool ("enable_transparency");
     config_setbool ("enable_transparency", status);
+    tilda_window_apply_transparency (tw, status);
+}
+
+void tilda_window_refresh_transparency (tilda_window *tw)
+{
+    gboolean status = config_getbool ("enable_transparency");
+    tilda_window_apply_transparency (tw, status);
+}
+
+static void tilda_window_apply_transparency (tilda_window *tw, gboolean status)
+{
+    tilda_term *tt;
+    guint i;
+
 #ifdef VTE_290
     gdouble transparency_level = 0.0;
     transparency_level = ((gdouble) config_getint ("transparency"))/100;
@@ -568,7 +625,7 @@ static gboolean goto_tab_generic (tilda_window *tw, guint tab_number)
         goto_tab (tw, tab_number - 1);
     }
 
-    return TRUE;
+    return GDK_EVENT_STOP;
 }
 
 /* These all just call the generic function since they're all basically the same
@@ -602,8 +659,7 @@ static gint ccopy (tilda_window *tw)
     }
     vte_terminal_copy_clipboard (VTE_TERMINAL(list->data));
 
-    /* Stop the event's propagation */
-    return TRUE;
+    return GDK_EVENT_STOP;
 }
 
 static gint cpaste (tilda_window *tw)
@@ -623,8 +679,7 @@ static gint cpaste (tilda_window *tw)
     }
     vte_terminal_paste_clipboard (VTE_TERMINAL(list->data));
 
-    /* Stop the event's propagation */
-    return TRUE;
+    return GDK_EVENT_STOP;
 }
 
 /* Tie a single keyboard shortcut to a callback function */
@@ -671,7 +726,7 @@ static gint tilda_window_setup_keyboard_accelerators (tilda_window *tw)
     tilda_add_config_accelerator_by_path("copy_key",       "<tilda>/context/Copy",              G_CALLBACK(ccopy),                          tw);
     tilda_add_config_accelerator_by_path("paste_key",      "<tilda>/context/Paste",             G_CALLBACK(cpaste),                         tw);
     tilda_add_config_accelerator_by_path("fullscreen_key", "<tilda>/context/Toggle Fullscreen", G_CALLBACK(toggle_fullscreen_cb),           tw);
-    tilda_add_config_accelerator_by_path("quit_key",       "<tilda>/context/Quit",              G_CALLBACK(gtk_main_quit),                  tw);
+    tilda_add_config_accelerator_by_path("quit_key",       "<tilda>/context/Quit",              G_CALLBACK(tilda_window_confirm_quit),      tw);
     tilda_add_config_accelerator_by_path("toggle_transparency_key", "<tilda>/context/Toggle Transparency", G_CALLBACK(toggle_transparency_cb),      tw);
     tilda_add_config_accelerator_by_path("toggle_searchbar_key", "<tilda>/context/Toggle Searchbar", G_CALLBACK(tilda_window_toggle_searchbar),     tw);
 
@@ -799,7 +854,10 @@ static gboolean delete_event_callback (G_GNUC_UNUSED GtkWidget *widget,
     return FALSE;
 }
 
-gboolean search_box_key_cb (GtkWidget *widget, GdkEvent  *event, tilda_window *tw) {
+static gboolean search_box_key_cb (GtkWidget *widget,
+                                   GdkEvent  *event,
+                                   tilda_window *tw)
+{
     GdkEventKey *event_key = (GdkEventKey*)event;
     if (event_key->keyval == GDK_KEY_Return) {
         tilda_window_search(widget, tw, FALSE);
@@ -818,8 +876,9 @@ gboolean search_box_key_cb (GtkWidget *widget, GdkEvent  *event, tilda_window *t
     return GDK_EVENT_PROPAGATE;
 }
 
-gboolean entry_search_text_changed_callback (G_GNUC_UNUSED GtkEditable *editable,
-                                             tilda_window *tw) {
+static gboolean entry_search_text_changed_callback (G_GNUC_UNUSED GtkEditable *editable,
+                                                    tilda_window *tw)
+{
     gtk_widget_hide (tw->search->label_search_end);
     tw->search->is_search_result = TRUE;
 
@@ -860,17 +919,17 @@ static void page_reordered_cb (GtkNotebook  *notebook,
                         tilda_window *tw) {
     DEBUG_FUNCTION ("page_reordered_cb");
     GList *terminals;
-    tilda_term *tilda_term;
+    tilda_term *terminal;
     guint i;
 
     terminals = tw->terms;
 
     for (i = 0; i < g_list_length (terminals); i++)
     {
-        tilda_term = g_list_nth_data (terminals, i);
-        if (tilda_term->hbox == child) {
-            terminals = g_list_remove (terminals, tilda_term);
-            tw->terms = g_list_insert (terminals, tilda_term, page_num);
+        terminal = g_list_nth_data (terminals, i);
+        if (terminal->hbox == child) {
+            terminals = g_list_remove (terminals, terminal);
+            tw->terms = g_list_insert (terminals, terminal, page_num);
             break;
         }
     }
@@ -1099,22 +1158,32 @@ gint tilda_window_add_tab (tilda_window *tw)
     }
 
     /* Create page and append to notebook */
-    label = gtk_label_new ("Tilda");
+    label = gtk_label_new (config_getstr("title"));
     /* Strangely enough, prepend puts pages on the end */
     index = gtk_notebook_append_page (GTK_NOTEBOOK(tw->notebook), tt->hbox, label);
     gtk_notebook_set_current_page (GTK_NOTEBOOK(tw->notebook), index);
     gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK(tw->notebook), tt->hbox, TRUE);
 
-    /* We should show the tabs if there are more than one tab in the notebook,
+    if(config_getbool ("expand_tabs")) {
+        gtk_container_child_set (GTK_CONTAINER(tw->notebook),
+            GTK_WIDGET(tt->hbox),
+            "tab-expand", TRUE,
+            "tab-fill", TRUE,
+            NULL);
+    }
+
+    /* We should show the tabs if there are more than one tab in the notebook
+     * (or show single tab is set),
      * and tab position is not set to hidden */
-    if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook)) > 1 &&
+    if ((gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook)) > 1 ||
+            config_getbool("show_single_tab")) &&
             config_getint("tab_pos") != NB_HIDDEN)
         gtk_notebook_set_show_tabs (GTK_NOTEBOOK (tw->notebook), TRUE);
 
     /* The new terminal should grab the focus automatically */
     gtk_widget_grab_focus (tt->vte_term);
 
-    return TRUE; //index;
+    return GDK_EVENT_STOP; //index;
 }
 
 gint tilda_window_close_tab (tilda_window *tw, gint tab_index, gboolean force_exit)
@@ -1139,7 +1208,8 @@ gint tilda_window_close_tab (tilda_window *tw, gint tab_index, gboolean force_ex
     gtk_notebook_remove_page (GTK_NOTEBOOK (tw->notebook), tab_index);
 
     /* We should hide the tabs if there is only one tab left */
-    if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook)) == 1)
+    if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook)) == 1 &&
+            !config_getbool("show_single_tab"))
         gtk_notebook_set_show_tabs (GTK_NOTEBOOK (tw->notebook), FALSE);
 
     /* With no pages left, either leave the program or create a new
@@ -1193,5 +1263,24 @@ gint tilda_window_close_tab (tilda_window *tw, gint tab_index, gboolean force_ex
     /* Free the terminal, we are done with it */
     tilda_term_free (tt);
 
-    return TRUE;
+    return GDK_EVENT_STOP;
+}
+
+gint tilda_window_confirm_quit (tilda_window *tw)
+{
+    DEBUG_FUNCTION(__FUNCTION__);
+
+    gboolean can_quit = TRUE;
+
+    if(config_getbool("prompt_on_exit")) {
+        char * message = _("Are you sure you want to Quit?");
+
+        can_quit = show_confirmation_dialog (tw, message);
+    }
+
+    if (can_quit) {
+        gtk_main_quit ();
+    }
+
+    return GDK_EVENT_STOP;
 }
