@@ -163,6 +163,11 @@ gboolean tilda_window_init (const gchar *config_file, const gint instance, tilda
  */
 gint tilda_window_free (tilda_window *tw);
 
+/**
+ * Applies or reapplies the current fullscreen state of the tilda window.
+ */
+void tilda_window_set_fullscreen(tilda_window *tw);
+
 /**
  * This toggles the fullscreen mode on or off. This is intended to be registered
  * as a GCallback in order to be invoked after some user action.
@@ -190,11 +195,21 @@ gboolean tilda_window_update_keyboard_accelerators (const gchar* path, const gch
  */
 void tilda_window_toggle_transparency(tilda_window *tw);
 
+/**
+ * Refreshes transparency
+ */
+void tilda_window_refresh_transparency(tilda_window *tw);
+
 /**
  * Toggles the search bar of the tilda window.
  */
 gint tilda_window_toggle_searchbar (tilda_window *tw);
 
+/**
+ * Show confirm dialog before quitting (if enabled)
+ */
+gint tilda_window_confirm_quit (tilda_window *tw);
+
 #define TILDA_WINDOW(data) ((tilda_window *)(data))
 
 /* Allow scales a bit smaller and a bit larger than the usual pango ranges */
